# Supported Switch model: Cisco Nexus7000

import os
import datetime
from openpyxl import *
from parse import *
from RunSwitch import *


def get_command(fname):
    #f=open('nxos_cmd.txt', 'r')
    try:
        f=open(fname, 'r')
    except:
        print("File '%s' not found" %(fname))
        exit()
    s=f.read()
    cmdlist=s.split('\n')
    return cmdlist

def get_rawdata(device):
    clist=get_command('nxos_cmd.txt')
    rawdata='IPADDRESS '+device[0]+' \n'
    cmd='term length 0\n'
    for c in clist:
        cmd=cmd+c+'\n'
    rawdata=rawdata+runTELNET(device, cmd)
    return rawdata

def save_rawdata(rawdata):
    l=rawdata.split('\n')
    if len(rawdata)!=0:
        hostname=hname(l).strip()
    else:
        hostname='error'
    fname='rawdata/'+hostname+'_raw.txt'
    f=open(fname, 'w')
    f.write(rawdata)
    f.close()

def make_report(docname, sheetname, rawdata):
    #wb=WorkBook()
    #ws=wb.active
    l=rawdata.split('\n')
    wb=load_workbook(docname)
    source_ws=wb['template_nxos']
    target_ws=wb.copy_worksheet(source_ws)
    ws=target_ws
    ws.title=sheetname
    now=datetime.datetime.now()
    hostname=hname(l)
    ip=ipaddr(l)
    # Raw 데이타로부터 점검값을 추출한다.
    modl=model(l)
    tem=temperature(l)
    fan=fanstate(l)
    pow=powersupply(l)
    upt=uptime(l)
    cpu=cpuutil(l)
    mem=memutil(l)
    mod=module(l)
    ints=iface(l)
    inte=ifaceerr(l)
    rou=chkroute(l)
    ckl=chklog(l)

    ws['G4']='%s년 %s월 %s일' %(now.year, now.month, now.day)
    ws['G6']=hostname
    ws['G5']=modl
    ws['G7']=ip
# 보고서에 점검값을 출력.
    ws['N12']=tem[0]
    ws['N13']=fan[0]
    ws['N14']=pow[0]
    ws['N17']=upt[0]
    ws['N18']=cpu[0]
    ws['N19']=mem[0]
    ws['N22']=mod[0]
    ws['N25']=ints[0]
    ws['N26']=inte[0]
    ws['N29']=rou[0]
    ws['N32']=ckl[0]
# 보고서에 점검결과 이상 유무를 출력.
    ws['W12']=tem[1]
    ws['W13']=fan[1]
    ws['W14']=pow[1]
    ws['W17']=upt[1]
    ws['W18']=cpu[1]
    ws['W19']=mem[1]
    ws['W22']=mod[1]
    ws['W25']=ints[1]
    ws['W26']=inte[1]
    ws['W29']=rou[1]
    ws['W32']=ckl[1]

    wb.save(docname)

if __name__=='__main__':
    #make_report('report.xlsx', '대학본부 4507')
    #s=get_rawdata(['165.132.210.1','23','wroot','n@m17P$&iu','telnet','cisco'])
    #save_rawdata(s)
    #makeReport('report.xlsx', '대학본부', s)
    f=open('switch.txt', 'r')
    s=f.read()
    f.close()

    print('Now, making report...\n')
    l=s.split('\n')
    for i in l:
        dev=' '.join(i.split())
        device=dev.split(' ')
        s=get_rawdata(device)
        #ip=' IPADDRESS '+device[0]+'\n'
        #print(ip)
        #s=ip+s
        save_rawdata(s)
        t=s.split('\n')
        hostname=hname(t)
        print(hostname)
        make_report('report.xlsx', hostname, s)
    print('Well DONE!!!')
